package com.sunyard.ssp.common.exception;

import com.sunyard.chsm.model.R;
import com.sunyard.chsm.utils.ThrowableUtils;
import com.sunyard.ssp.common.Result;
import com.sunyard.ssp.utils.ResultUtil;
import lombok.extern.slf4j.Slf4j;
import org.mybatis.spring.MyBatisSystemException;
import org.springframework.validation.BindException;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import java.util.Arrays;
import java.util.IllegalFormatException;
import java.util.List;
import java.util.Optional;

/**
 *  全局controller异常处理
 * @auther: ZS
 * @description:
 * @param:
 * @return:
 * @date: 2020/5/25 16:24
 */
@Slf4j
@RestControllerAdvice
public class GlobalExceptionResolver {

    /**
     * 处理所有不可知异常
     *
     * @param e 异常
     * @return json结果
     */
//    @ExceptionHandler(Exception.class)
//    @ResponseBody
//    public Result<Object> handleException(Exception e) {
//        // 打印异常堆栈信息
//        return new ResultUtil().setErrorMsg(e.getMessage());
//    }

    /**
     * 数据持久层
     */
    @ExceptionHandler(MyBatisSystemException.class)
    @ResponseBody
    public Result<Object> handleMyBatisSystemException(MyBatisSystemException e) {
        log.info("数据持久层");
        return new ResultUtil().setErrorMsg(e.getLocalizedMessage());
    }

    /**
     * 校验器
     */
    @ExceptionHandler(BindException.class)
    @ResponseBody
    public Result<Object> handleBindException(BindException e) {
        log.info("校验器");
        FieldError fieldError = e.getBindingResult().getFieldError();
        return new ResultUtil().setErrorMsg(fieldError.getDefaultMessage());
    }

    @ExceptionHandler(MethodArgumentNotValidException.class)
    public R<?> validExceptionHandler(MethodArgumentNotValidException ex) {
        return Optional.of(ex)
                .map(MethodArgumentNotValidException::getFieldError)
                .map(FieldError::getDefaultMessage)
                .map(message -> R.error(400, message))
                .get();
    }

    private static final List<Class<? extends Exception>> EX_CLASS = Arrays.asList(
            IllegalArgumentException.class, IllegalStateException.class,
            IllegalFormatException.class,
            SspwebException.class,
            UnsupportedOperationException.class);

    @ExceptionHandler(Exception.class)
    public R<?> exceptionHandler(Exception ex) {
        String errorMessage = ThrowableUtils.buildErrorMessage(ex);
        SspwebException serviceException = ThrowableUtils.findException(SspwebException.class, ex);
        if (serviceException != null) {
            log.warn("Validation failed -> {}, errorMessage: \n {}", serviceException.getMessage(), errorMessage);
            return R.error(400, serviceException.getMessage());
        }

        for (Class<? extends Exception> eClass : EX_CLASS) {
            Exception exception = ThrowableUtils.findException(eClass, ex);
            if (exception == null) {
                continue;
            }
            log.warn("Validation failed -> {} ", errorMessage);
            return R.error(400, exception.getMessage());
        }
        log.error("系统异常 -> ", ex);
        return R.error("系统异常");
    }

}
